<?php
/***************************************************************************
 * 
 * Copyright (c) 2017 Baidu.com, Inc. All Rights Reserved
 * 
 **************************************************************************/
 
/**
 * @file pushdata.php
 * @brief 推送数据
 **/

require_once('http_lib.php');
date_default_timezone_set('Asia/Shanghai');

class PushData {
    //log 路径
    private $log_prefix = 'log';

    //推送的数据量最大值
    private $DATA_MAX_SIZE = 10485760;

    //推送超时，重试次数
    private $retry = 3;

    //推送接口
    private $url = '';

    /**
     * PushData constructor.
     * @param $url 推送接口
     */
    function __construct($url){
        if (!is_dir($this->log_prefix)) {
            mkdir($this->log_prefix);
        }
        $this->url = $url;
    }

    /**
     * @param $items array
     * @param $type string 日志类型 NOTICE WARNING ERROR
     */
    function log($items, $type = 'NOTICE') {
        $log_file = strftime('%Y-%m-%d', time());
        $log_path = $this->log_prefix . DIRECTORY_SEPARATOR . $log_file;

        $log_str = $type . ' ' . strftime('%Y-%m-%d %H:%M:%S', time()) . ' ';

        foreach ($items as $key => $value) {
            $log_str .= "\x01" . $key . "\x02" . $value;
        }
        $log_str .= "\n";
        file_put_contents($log_path, $log_str, FILE_APPEND | LOCK_EX);
    }

    /**
     * @return $rid string
     * @desc 生成一个相对唯一的request id，20位。方便定位问题
     */
    function gen_rid() {
        return sprintf("%s%.6d",strftime('%Y%m%d%H%M%S', time()), rand(1, 100000));
    }

    /**
     * @param $url string 推送接口
     * @param $data string 推送数据
     * @param $opt array http_post参数
     * @param $retry int 重试次数
     * @param $rid string 请求id
     * @return int
     */
    function post_retry($url, $data, $opt, $retry, $rid) {
        $res = http_post($url, $data, $opt);
        $ret = 0;
        if (($res['error_no'] == 0)) {
            $resp_arr = json_decode($res['data'], true);

            if (($res['http_code'] == 503) || ($res['http_code'] == 502) ) {
                //连接超时或者达到并发限制，则进行重试
                $ret = -1;
            }

            //对http_code 是否为200日志方式不同
            if (($res['http_code'] == 200) && ($resp_arr["code"] == 200)) {
                $log_data = [
                    'rid' => $rid,
                    'server_logid' => $resp_arr['id'],
                    'recv_num' => $resp_arr['recv_num'],
                    'msg' => $res['data'],
                ];
                $this->log($log_data);
            } else {
                $log_data = [
                    'rid' => $rid,
                    'retry' => $this->retry - $retry + 1,
                    'http_code' => $res['http_code'],
                    'error' => str_replace(array("\r\n", "\n", "\r"), "", $res['data']),
                ];
                $this->log($log_data, 'WARNING');
            }
        } else {
            $log_data = [
                'rid' => $rid,
                'error' => $res['msg'],
            ];
            $this->log($log_data, 'ERROR');
        }
        return $ret;
    }

    /**
     * @param $url string 推送接口
     * @param $post_data array 推送数据
     * @return int
     */
    function post_json_data($post_data) {
        //对请求的接口加一个请求id，方便定位问题
        $rid = self::gen_rid();
        $url = $this->url . "&rid" . $rid;

        if (is_array($post_data)) {
            $data = implode($post_data, "\n");
        } else {
            //如果不是array类型，退出推送
            $log_data = [
                'rid' => $rid,
                'msg' => '$post_data需要为array类型',
            ];
            $this->log($log_data, 'ERROR');
            return -1;
        }

        //记录当前请求记录
        $log_data = [
            'rid' => $rid,
            'send_size' => strlen($data),
            'send_num' => count($post_data),
        ];
        $this->log($log_data);

        //数据量大于限制值的包不发送
        if (strlen($data) > $this->DATA_MAX_SIZE) {
            $log_data = [
                'rid' => $rid,
                'msg' => '请求包大小应小于10M',
            ];
            $this->log($log_data, 'ERROR');
            return -1;
        }

        $opt = [
            'content-type' => 'application/json',
            'timeout' => 3,
        ];

        $retry = $this->retry;
        while ($retry) {
            $ret = $this->post_retry($url, $data, $opt, $retry, $rid);
            $retry = $retry - 1;
            if ($ret == 0) {
                break;
            }
        }
    }

};

